/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.syncclient.demo;

import java.util.Date;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import sync4j.syncclient.spdm.DMException;
import sync4j.syncclient.spdm.SimpleDeviceManager;
import sync4j.syncclient.spdm.ManagementNode;

import sync4j.foundation.pdi.utils.TimeUtils;


/**
 * Fields helper methods.
 *
 * @author Fabio Maggi       @ Funambol
 * @author Marco Magistrali  @ Funambol
 */

public abstract class FieldsHelper
implements ConfigurationParameters {

    //--------------------------------------------------------------- Constants

    public static final String DATE_FORMAT             = "dd/MM/yyyy"          ;
    public static final String DATE_FORMAT_AND_TIME    = "dd/MM/yyyy HH:mm:ss" ;
    public static final String UTCDATE_FORMAT          = "yyyyMMdd'T'HHmmss"   ;

    //--------------------------------------------------------------- Private data

    private static Language ln = new Language() ;

    //--------------------------------------------------------------- Public methods

    //--------------------------------------------------------------- Protected methods

    protected static boolean checkDate (String date, String dateFormat) {

        StringTokenizer  st        = null ;
        SimpleDateFormat formatter = null ;
        Date dTemp                 = new Date();
        int k = 0;

        if (date != null && date.length() > 0)  {

            try {
                if (dateFormat.equals(DATE_FORMAT)) {
                    formatter = new SimpleDateFormat (DATE_FORMAT);
                }
                else if (dateFormat.equals(DATE_FORMAT_AND_TIME)) {
                    formatter = new SimpleDateFormat (DATE_FORMAT_AND_TIME);
                }
                formatter.setLenient   (false );
                dTemp = formatter.parse (date  );

                if (dateFormat.equals(DATE_FORMAT)) {
                    st = new StringTokenizer(date, "/");
                    while (st.hasMoreTokens()) {

                        k = Integer.parseInt(st.nextToken());

                        if (k >9999) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }

                    }
                } else if (dateFormat.equals(DATE_FORMAT_AND_TIME)) {
                    String sTemp =  date.substring(0, 10);
                    st = new StringTokenizer(sTemp, "/");
                    while (st.hasMoreTokens()) {

                        k = Integer.parseInt(st.nextToken());

                        if (k >9999) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }

                    }
                    sTemp = date.substring(11);

                    st = new StringTokenizer(sTemp, ":");
                    int i = 0;
                    while (st.hasMoreTokens()) {
                        String s = st.nextToken();

                        if (s.length() > 2) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }

                        k = Integer.parseInt(s);

                        // HH
                        if (i == 0 && (k < 0 || k > 24)) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }
                        // MM
                        if (i == 1 && (k < 0 || k > 60)) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }
                        // ss
                        if (i == 2 && (k < 0 || k > 60)) {
                            throw new NumberFormatException(ln.
                                    getString ("error_number_format"));
                        }

                        i++;
                    }
                }
            } catch   (NullPointerException  e ) {
               return false;
            } catch   (ParseException        e ) {
               return false;
            } catch   (NumberFormatException e ) {
               return false;
            } catch   (Exception             e ) {
               return false;
            }

        }

        return true;

    }

    protected static String convertDateToUTC(String stringDate) {

        ManagementNode rootNode   = null ;
        Hashtable      parameters = null;

        String    adjustDate      = null;

        if (stringDate == null || stringDate.length() == 0) {
            return "";
        }

        rootNode = SimpleDeviceManager.getDeviceManager().getManagementTree();

        try {
            parameters = rootNode.getNodeValues(MainWindow.DM_VALUE_PATH);
        } catch (DMException e) {
            e.printStackTrace();
        }

        adjustDate = (String) parameters.get( PARAM_ADJUSTDATE );

        if ("true".equals(adjustDate)) {
            return TimeUtils.localTimeToUTC         (stringDate ) ;
        } else {
            return convertDateToOnlyFormatUTC       (stringDate ) ;
        }
    }

    protected static String convertDateFromUTC (String stringDate) {

        ManagementNode rootNode     = null ;
        Hashtable      parameters   = null ;

        String         adjustDate   = null ;

        if (stringDate == null || stringDate.length() == 0) {
            return "";
        }

        rootNode = SimpleDeviceManager.getDeviceManager().getManagementTree();

        try {
            parameters = rootNode.getNodeValues(MainWindow.DM_VALUE_PATH);
        } catch (DMException e) {
            e.printStackTrace();
        }

        adjustDate = (String) parameters.get(PARAM_ADJUSTDATE);

        if ("true".equals(adjustDate)) {
            return TimeUtils.UTCToLocalTime         (stringDate ) ;
        } else {
            return convertDateFromOnlyFormatUTC     (stringDate ) ;
        }

    }

    //--------------------------------------------------------------- Private methods

    private static String convertDateToOnlyFormatUTC (String stringDate) {

        String            ret        = stringDate ;
        SimpleDateFormat  formatter  = null       ;
        StringTokenizer   st         = null       ;
        Date              date       = null       ;

        try {
            if (stringDate != null && stringDate.length() > 10) {
                formatter = new SimpleDateFormat (DATE_FORMAT_AND_TIME);
            }
            else {
                formatter = new SimpleDateFormat (DATE_FORMAT);
            }
            formatter.setLenient(false);
            date = formatter.parse(stringDate);

            formatter.applyPattern (UTCDATE_FORMAT ) ;
            ret = formatter.format (date           ) ;

        } catch (Exception e) {
            e.printStackTrace();
        }

        return ret;
    }

    private static String convertDateFromOnlyFormatUTC (String stringDate) {

        String            ret        = stringDate ;
        SimpleDateFormat  formatter  = null       ;
        StringTokenizer   st         = null       ;
        Date              date       = null       ;

        try {

            formatter = new SimpleDateFormat (UTCDATE_FORMAT);
            formatter.setLenient(false);
            date = formatter.parse (stringDate);

            formatter.applyPattern (DATE_FORMAT_AND_TIME ) ;
            ret = formatter.format (date        ) ;

        } catch (Exception e) {
           e.printStackTrace();
        }

        return ret;

    }

}